#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc. 
#

import sys,os,re
from rpmUtils.arch import getBaseArch
from socket import getfqdn,gethostbyname

def get_fingerprints():
    '''Return dict of machine's fingerprints (mac addresses currently)'''
    nics =  get_nics_from_ip()
    return [v for v in nics.values()]

def get_ksdevice(server):
    '''Get mac address of kickstart device'''
    ksdevice=''
    nics = get_nics_from_ip()
    address = gethostbyname(server.replace('http://','').split(':')[0])
    pattern = re.compile('^.+\\b(dev)\\b +\\b([a-zA-Z0-9]+)\\b.+$')
    f = os.popen('LC_ALL=C /sbin/ip route get %s' % address)
    l1 = f.readline().strip()
    f.close()
    iface = re.match(pattern, l1).group(2)
    if nics.has_key(iface):
        ksdevice = nics[iface]
    return ksdevice

def get_nics_from_ip():
    '''Return dict {network interface:mac address} using ip command'''
    nics = {}
    # we don't care about vmnet? peth? vif? veth? xenbr? pan? interfaces
    pattern1 = re.compile('^.+\\b(?:vmnet|peth|vif|veth|xenbr|virbr|pan).+\\b.+$')
    pattern2 = re.compile('^.*\\blink/ether\\b.+$')
    f = os.popen('LC_ALL=C /sbin/ip link list')
    line1 = f.readline().strip()
    line2 = f.readline().strip()
    while line1 and line2:
        if not re.match(pattern1, line1) and re.match(pattern2, line2):
            # network interface == first line, seond word, no ':'
            iface = line1.split(' ')[1].strip(':')
            # mac address == second line, second word, uppercase
            mac = line2.split(' ')[1].upper()
            nics[iface] = mac
        line1 = f.readline().strip()
        line2 = f.readline().strip()
    return nics

def get_procinfo():
    ret = {}
    for k,v in {'pciinfo':'/proc/bus/pci/devices','cpuinfo':'/proc/cpuinfo',\
                'meminfo':'/proc/meminfo','sysinfo':'/proc/sysinfo'}.items():
        if not os.path.exists(v):
            continue
        else:
            ret[k] = open(v,'r').readlines()
    return ret

def get_machineinfo(server):
    ret = {}
    ret['arch'] = getBaseArch()
    ret['fqdn'] = getfqdn()
    ret['fingerprints'] = get_fingerprints()
    ret['ksdevice'] = get_ksdevice(server)
    for k,v in get_procinfo().items():
        ret[k] = v
    return ret

def isParaVirt():
    """
    This function determines if the running system is a paravirt guest.
    This code is borrowed heavily from rhnreg.py (packaged with rhn-setup).
       2.  Check /sys/hypervisor/uuid.  If exists and is non-zero, we know
           the system is a para-virt guest; exit.
    """

    isGuest = False
    if os.path.exists("/proc/xen/xsd_port"):
        return False
    if os.path.exists("/sys/hypervisor/uuid"):
        try:
            uuid_file = open('/sys/hypervisor/uuid', 'r')
            uuid = uuid_file.read()
            uuid_file.close()
            if len(uuid.strip("0-\n")) != 0:
                isGuest = True
        except IOError:
            # Failed; must not be para-virt.
            pass

    return isGuest
